@@ -1,19 +1,18 @@
 from multisensor_pipeline import BaseProcessor
-from multisensor_pipeline.dataframe.dataframe import MSPDataFrame
-from typing import Optional
+from multisensor_pipeline.dataframe.dataframe import MSPDataFrame, Topic
+from typing import Optional, List
 import logging
 import numpy as np
-
+from copy import copy
 
 logger = logging.getLogger(__name__)
 
 
 class DownsamplingProcessor(BaseProcessor):
-
     class DataFrameHistory:
 
-        def __init__(self, topic_uid, fps_out, window_size=5, interpolation=None):
-            self.topic_uid = topic_uid
+        def __init__(self, topic_uuid, fps_out, window_size=5, interpolation=None):
+            self.topic_uid = topic_uuid
             self.target_fps = fps_out
             self.window_size = window_size  # TODO: expose window_size, mainly for interpolation
             self.interpolation = interpolation  # TODO: utilize interpolation for, e.g., averaging the data
@@ -86,35 +85,43 @@ class DownsamplingProcessor(BaseProcessor):
         def period_time_in(self):
             return (self.dataframes[-1].timestamp - self.dataframes[0].timestamp) / len(self.dataframes)
 
-    def __init__(self, topic_names=None, sampling_rate=5):
+    def __init__(self, target_topics: Optional[List[Topic]] = None, samplerate: int = 5):
         """
         Downsamples a signal to a given sampling_rate [Hz], if the original rate is higher.
         Otherwise, the sampling rate stays the same (no upsampling).
-        @param topic_names: the dtype to be resampled; if None, all incoming dtypes are resampled
-        @param sampling_rate: the desired sampling rate [Hz]
+        @param target_topics: the dtype to be resampled; if None, all incoming dtypes are resampled
+        @param samplerate: the desired sampling rate [Hz]
         """
         super(DownsamplingProcessor, self).__init__()
-        self._topic_names = topic_names
-        self._sampling_rate = sampling_rate
-        self._period_time = 1. / sampling_rate
+        self._target_topics = target_topics if target_topics is not None else [Topic()]
+        self._sampling_rate = samplerate
+        self._period_time = 1. / samplerate
 
         self._sample_hist = dict()
         self._last_sent = dict()
         self._last_received = dict()
 
-    def _get_history(self, uid) -> DataFrameHistory:
-        if uid not in self._sample_hist:
-            self._sample_hist[uid] = self.DataFrameHistory(uid, fps_out=self._sampling_rate)
-        return self._sample_hist[uid]
+    def _get_history(self, uuid) -> DataFrameHistory:
+        if uuid not in self._sample_hist:
+            self._sample_hist[uuid] = self.DataFrameHistory(uuid, fps_out=self._sampling_rate)
+        return self._sample_hist[uuid]
 
     def on_update(self, frame: MSPDataFrame) -> Optional[MSPDataFrame]:
-        if self._topic_names is None or frame.topic.name in self._topic_names:
+        if self._target_topics is None or frame.topic in self._target_topics:
             hist = self._get_history(frame.topic.uuid)
             hist.add(frame)
             _frame = hist.get_dataframe()
             if _frame is not None:
-                _topic = self._generate_topic(name=f"{frame.topic.name}.{self._sampling_rate}Hz",
-                                              dtype=frame.topic.dtype)
-                _frame.topic = _topic
-                return _frame
+                _topic = Topic(name=f"{frame.topic.name}.{self._sampling_rate}Hz",
+                               dtype=frame.topic.dtype)
+                new_frame = copy(_frame)
+                new_frame.topic = _topic
+                return new_frame
 
+    @property
+    def input_topics(self) -> List[Topic]:
+        return self._target_topics
+
+    @property
+    def output_topics(self) -> Optional[List[Topic]]:
+        return [Topic(name=f"{t.name}.{self._sampling_rate}Hz", dtype=t.dtype) for t in self._target_topics]
